#ifndef C_P1CAMERA_COMMON_STRUCTS_H
#define C_P1CAMERA_COMMON_STRUCTS_H

/**
* @file P1CameraSdkCommonStructs.h
* @brief This file contains struct types that a used by higher layer APIs, like C++, C# and Python
*        Types defined here are include in the public headers for our C++ API layer.
*/

#include <stdint.h>
#include <string.h>	// For memcmp()
#include <stdexcept>

#include "C_P1CameraCommonDefs.h"	// For CameraId
#include "C_P1CameraCommonProperties.h"	// For PropertyId
#include "C_P1CameraCommonEvents.h"	// For CameraEventId
#include "C_P1CameraCommonNotificationEvent.h" // For NotificationEvent
#include "C_P1CameraCommonLinkage.h"

namespace P1
{
namespace CameraSdk
{
#pragma pack(push, 8)

	struct EXPORT_API Version
	{
		uint32_t major = 0;
		uint32_t minor = 0;
		uint32_t build = 0;
	};

	struct EXPORT_API ValueColorFloat
	{
		double alpha = 0.0;
		double red = 0.0;
		double green = 0.0;
		double blue = 0.0;

		bool operator==(const ValueColorFloat& other) const
		{
			return alpha == other.alpha && red == other.red && green == other.green && blue == other.blue;
		}
	};

	struct EXPORT_API ValueArea
	{
		int32_t width = 0;
		int32_t height = 0;

		bool operator==(const ValueArea& other) const
		{
			return width == other.width && height == other.height;
		}
	};

	struct EXPORT_API ValueAreaFloat
	{
		double width = 0;
		double height = 0;

		bool operator==(const ValueAreaFloat& other) const
		{
			return std::abs(width - other.width) < 0.001 && std::abs(height - other.height) < 0.001;
		}
	};

	struct EXPORT_API ValuePoint
	{
		int32_t x = 0;
		int32_t y = 0;

		bool operator==(const ValuePoint& other) const
		{
			return x == other.x && y == other.y;
		}
	};

	struct EXPORT_API ValueRect
	{
		ValuePoint pos;
		ValueArea  size;

		bool operator==(const ValueRect& other) const
		{
			return pos == other.pos && size == other.size;
		}
	};

	struct EXPORT_API ValueRange
	{
		int64_t min;
		int64_t max;

		bool operator==(const ValueRange& other) const
		{
			return min == other.min && max == other.max;
		}
	};

	struct EXPORT_API ValueRangeFloat
	{
		double min;
		double max;

		bool operator==(const ValueRangeFloat& other) const
		{
			return std::abs(min - other.min) < 0.001 && std::abs(max - other.max) < 0.001;
		}
	};

    // NotificationEvent moved to its own file


#pragma pack(pop)
}	// CameraSdk
}	// P1

#endif // C_P1CAMERA_COMMON_STRUCTS_H
