#ifndef P1CAMERA_IMAGE_CHUNK_HPP
#define P1CAMERA_IMAGE_CHUNK_HPP

#include <cstdint>
#include <memory>   // For std::shared_ptr
#include "C_P1CameraCommonLinkage.h"

namespace P1
{
namespace CameraSdk
{
    // ----------------------------------------------------------------------
    // An interface for an allocator.    
    template <class T>
    struct EXPORT_API IAllocator
    {
        virtual T* Allocate(uint32_t size) = 0;
        virtual void Deallocate(T* data) = 0;
    };
    
    // ----------------------------------------------------------------------
    // A simple straight-forward implementation of an IAllocator of uint8_t's
    // using new and delete.
    struct EXPORT_API SimpleBufferAllocator : IAllocator<uint8_t>
    {
        virtual uint8_t* Allocate(uint32_t size) override;
        virtual void Deallocate(uint8_t* data) override;
    };

    // ----------------------------------------------------------------------
    // An attempt to provide a more memory safe buffer-type using smart-pointer.
    struct EXPORT_API SmartDataChunk
    {
        std::shared_ptr<uint8_t> mData = nullptr;
        uint32_t mSize = 0;
        
        // Meta information
        uint32_t mDataId = 0;
        uint32_t mChunkOffset = 0;
        uint32_t mRemainingBytes = 0;   // How many bytes are remaining for the this data-id
    };
    
    // ----------------------------------------------------------------------
    class Camera;
    //! zzz MEG 2020-02-11 08:57 Not currently used
    class EXPORT_API ImageChunk
    {
        friend class Camera;
    private:

        void* mImpl;

        ImageChunk& operator=(const ImageChunk& other) = delete;

        ImageChunk();
        ImageChunk(const ImageChunk& other) = delete;
        
    public:
        
        ~ImageChunk();

        ImageChunk(ImageChunk&& other);
        ImageChunk& operator=(ImageChunk&& other);

        virtual uint32_t ChunkNumber() const;
        virtual uint32_t TotalChunkCount() const;
        virtual uint32_t ChunkByteSize() const;
        virtual uint64_t ImageTotalByteSize() const;
        virtual uint8_t* Data();
    };
} // CameraSdk
} // P1

#endif  // P1CAMERA_IMAGE_CHUNK_HPP
